@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2020	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -34,7 +34,7 @@
 #include <netinet/tcp.h>
 #endif
 #include <netdb.h>
-
+#include <time.h>
 #include <stdio.h>
 #include <errno.h>
 #include <signal.h>
@@ -48,6 +48,12 @@
 #include "pool_config.h"
 #include "utils/elog.h"
 #include "utils/memutils.h"
+#include "protocol/pool_connection_pool.h"
+#include "protocol/pool_process_query.h"
+#include "protocol/pool_pg_utils.h"
+#include "main/pool_internal_comms.h"
+
+
 #include "context/pool_process_context.h"
 
 static int	pool_index;			/* Active pool index */
@@ -61,8 +67,6 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p);
 static int	check_socket_status(int fd);
 static bool connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool retry);
 
-#define TMINTMAX 0x7fffffff
-
 /*
 * initialize connection pools. this should be called once at the startup.
 */
@@ -111,18 +115,18 @@ pool_get_cp(char *user, char *database, int protoMajor, int check_socket)
 
 	for (i = 0; i < pool_config->max_pool; i++)
 	{
-		if (MASTER_CONNECTION(connection_pool) &&
-			MASTER_CONNECTION(connection_pool)->sp &&
-			MASTER_CONNECTION(connection_pool)->sp->major == protoMajor &&
-			MASTER_CONNECTION(connection_pool)->sp->user != NULL &&
-			strcmp(MASTER_CONNECTION(connection_pool)->sp->user, user) == 0 &&
-			strcmp(MASTER_CONNECTION(connection_pool)->sp->database, database) == 0)
+		if (MAIN_CONNECTION(connection_pool) &&
+			MAIN_CONNECTION(connection_pool)->sp &&
+			MAIN_CONNECTION(connection_pool)->sp->major == protoMajor &&
+			MAIN_CONNECTION(connection_pool)->sp->user != NULL &&
+			strcmp(MAIN_CONNECTION(connection_pool)->sp->user, user) == 0 &&
+			strcmp(MAIN_CONNECTION(connection_pool)->sp->database, database) == 0)
 		{
 			int			sock_broken = 0;
 			int			j;
 
 			/* mark this connection is under use */
-			MASTER_CONNECTION(connection_pool)->closetime = 0;
+			MAIN_CONNECTION(connection_pool)->closetime = 0;
 			for (j = 0; j < NUM_BACKENDS; j++)
 			{
 				connection_pool->info[j].counter++;
@@ -243,7 +247,6 @@ pool_create_cp(void)
 	POOL_CONNECTION_POOL *oldestp;
 	POOL_CONNECTION_POOL *ret;
 	ConnectionInfo *info;
-	int		main_node_id;
 
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 
@@ -256,7 +259,7 @@ pool_create_cp(void)
 
 	for (i = 0; i < pool_config->max_pool; i++)
 	{
-		if (in_use_backend_id(p) < 0)	/* is this connection pool out of use? */
+		if (MAIN_CONNECTION(p) == NULL)
 		{
 			ret = new_connection(p);
 			if (ret)
@@ -274,25 +277,21 @@ pool_create_cp(void)
 	 * discard it.
 	 */
 	oldestp = p = pool_connection_pool;
-	closetime = TMINTMAX;
+	closetime = MAIN_CONNECTION(p)->closetime;
 	pool_index = 0;
 
 	for (i = 0; i < pool_config->max_pool; i++)
 	{
-		main_node_id = in_use_backend_id(p);
-		if (main_node_id < 0)
-			elog(ERROR, "no in use backend found");	/* this should not happen */
-
 		ereport(DEBUG1,
 				(errmsg("creating connection pool"),
 				 errdetail("user: %s database: %s closetime: %ld",
-						   CONNECTION_SLOT(p, main_node_id)->sp->user,
-						   CONNECTION_SLOT(p, main_node_id)->sp->database,
-						   CONNECTION_SLOT(p, main_node_id)->closetime)));
+						   MAIN_CONNECTION(p)->sp->user,
+						   MAIN_CONNECTION(p)->sp->database,
+						   MAIN_CONNECTION(p)->closetime)));
 
-		if (CONNECTION_SLOT(p, main_node_id)->closetime < closetime)
+		if (MAIN_CONNECTION(p)->closetime < closetime)
 		{
-			closetime = CONNECTION_SLOT(p, main_node_id)->closetime;
+			closetime = MAIN_CONNECTION(p)->closetime;
 			oldestp = p;
 			pool_index = i;
 		}
@@ -300,21 +299,18 @@ pool_create_cp(void)
 	}
 
 	p = oldestp;
-	main_node_id = in_use_backend_id(p);
-	if (main_node_id < 0)
-		elog(ERROR, "no in use backend found");	/* this should not happen */
 	pool_send_frontend_exits(p);
 
 	ereport(DEBUG1,
 			(errmsg("creating connection pool"),
 			 errdetail("discarding old %zd th connection. user: %s database: %s",
 					   oldestp - pool_connection_pool,
-					   CONNECTION_SLOT(p, main_node_id)->sp->user,
-					   CONNECTION_SLOT(p, main_node_id)->sp->database)));
+					   MAIN_CONNECTION(p)->sp->user,
+					   MAIN_CONNECTION(p)->sp->database)));
 
 	for (i = 0; i < NUM_BACKENDS; i++)
 	{
-		if (CONNECTION_SLOT(p, i) == NULL)
+		if (!VALID_BACKEND(i))
 			continue;
 
 		if (!freed)
@@ -364,14 +360,14 @@ pool_connection_pool_timer(POOL_CONNECTION_POOL * backend)
 	/* look for any other timeout */
 	for (i = 0; i < pool_config->max_pool; i++, p++)
 	{
-		if (!MASTER_CONNECTION(p))
+		if (!MAIN_CONNECTION(p))
 			continue;
-		if (!MASTER_CONNECTION(p)->sp)
+		if (!MAIN_CONNECTION(p)->sp)
 			continue;
-		if (MASTER_CONNECTION(p)->sp->user == NULL)
+		if (MAIN_CONNECTION(p)->sp->user == NULL)
 			continue;
 
-		if (p != backend && MASTER_CONNECTION(p)->closetime)
+		if (p != backend && MAIN_CONNECTION(p)->closetime)
 			return;
 	}
 
@@ -395,6 +391,8 @@ pool_backend_timer_handler(int sig)
 void
 pool_backend_timer(void)
 {
+#define TMINTMAX 0x7fffffff
+
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 	int			i,
 				j;
@@ -411,30 +409,30 @@ pool_backend_timer(void)
 
 	for (i = 0; i < pool_config->max_pool; i++, p++)
 	{
-		if (!MASTER_CONNECTION(p))
+		if (!MAIN_CONNECTION(p))
 			continue;
-		if (!MASTER_CONNECTION(p)->sp)
+		if (!MAIN_CONNECTION(p)->sp)
 			continue;
-		if (MASTER_CONNECTION(p)->sp->user == NULL)
+		if (MAIN_CONNECTION(p)->sp->user == NULL)
 			continue;
 
 		/* timer expire? */
-		if (MASTER_CONNECTION(p)->closetime)
+		if (MAIN_CONNECTION(p)->closetime)
 		{
 			int			freed = 0;
 
 			ereport(DEBUG1,
 					(errmsg("backend timer handler called"),
 					 errdetail("expire time: %ld",
-							   MASTER_CONNECTION(p)->closetime + pool_config->connection_life_time)));
+							   MAIN_CONNECTION(p)->closetime + pool_config->connection_life_time)));
 
-			if (now >= (MASTER_CONNECTION(p)->closetime + pool_config->connection_life_time))
+			if (now >= (MAIN_CONNECTION(p)->closetime + pool_config->connection_life_time))
 			{
 				/* discard expired connection */
 				ereport(DEBUG1,
 						(errmsg("backend timer handler called"),
 						 errdetail("expired user: \"%s\" database: \"%s\"",
-								   MASTER_CONNECTION(p)->sp->user, MASTER_CONNECTION(p)->sp->database)));
+								   MAIN_CONNECTION(p)->sp->user, MAIN_CONNECTION(p)->sp->database)));
 				pool_send_frontend_exits(p);
 
 				for (j = 0; j < NUM_BACKENDS; j++)
@@ -459,8 +457,8 @@ pool_backend_timer(void)
 			else
 			{
 				/* look for nearest timer */
-				if (MASTER_CONNECTION(p)->closetime < nearest)
-					nearest = MASTER_CONNECTION(p)->closetime;
+				if (MAIN_CONNECTION(p)->closetime < nearest)
+					nearest = MAIN_CONNECTION(p)->closetime;
 			}
 		}
 	}
@@ -580,7 +578,7 @@ connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool r
 	int			error;
 	socklen_t	socklen;
 
-	pool_set_nonblock(fd);
+	socket_set_nonblock(fd);
 
 	for (;;)
 	{
@@ -748,7 +746,7 @@ connect_with_timeout(int fd, struct addrinfo *walk, char *host, int port, bool r
 		break;
 	}
 
-	pool_unset_nonblock(fd);
+	socket_unset_nonblock(fd);
 	return true;
 }
 
@@ -870,7 +868,6 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 	int			active_backend_count = 0;
 	int			i;
 	bool		status_changed = false;
-	volatile BACKEND_STATUS	status;
 
 	MemoryContext oldContext = MemoryContextSwitchTo(TopMemoryContext);
 
@@ -895,8 +892,8 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 		 * that the local status is up, while the global status has been
 		 * changed to down by failover.
 		 */
-		status = BACKEND_INFO(i).backend_status;
-		if (status != CON_UP && status != CON_CONNECT_WAIT)
+		if (BACKEND_INFO(i).backend_status != CON_UP &&
+			BACKEND_INFO(i).backend_status != CON_CONNECT_WAIT)
 		{
 			ereport(DEBUG1,
 					(errmsg("creating new connection to backend"),
@@ -904,8 +901,7 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 							   i, BACKEND_INFO(i).backend_status)));
 
 			/* sync local status with global status */
-			*(my_backend_status[i]) = status;
-			my_master_node_id = Req_info->master_node_id;
+			*(my_backend_status[i]) = BACKEND_INFO(i).backend_status;
 			continue;
 		}
 
@@ -941,16 +937,16 @@ static POOL_CONNECTION_POOL * new_connection(POOL_CONNECTION_POOL * p)
 					/* set down status to local status area */
 					*(my_backend_status[i]) = CON_DOWN;
 
-					/* if master_node_id is not updated, then update it */
-					if (Req_info->master_node_id == i)
+					/* if main_node_id is not updated, then update it */
+					if (Req_info->main_node_id == i)
 					{
-						int			old_master = Req_info->master_node_id;
+						int			old_main = Req_info->main_node_id;
 
-						Req_info->master_node_id = get_next_master_node();
+						Req_info->main_node_id = get_next_main_node();
 
 						ereport(LOG,
-								(errmsg("master node %d is down. Update master node to %d",
-										old_master, Req_info->master_node_id)));
+								(errmsg("main node %d is down. Update main node to %d",
+										old_main, Req_info->main_node_id)));
 					}
 
 					/*
@@ -1041,19 +1037,31 @@ pool_pool_index(void)
 }
 
 /*
- * Return the first node id in use.
- * If no node is in use, return -1.
+ * send frontend exiting messages to all connections.  this is called
+ * in any case when child process exits, for example failover, child
+ * life time expires or child max connections expires.
  */
-int
-in_use_backend_id(POOL_CONNECTION_POOL *pool)
-{
-	int	i;
 
-	for (i = 0; i < NUM_BACKENDS; i++)
+void
+close_all_backend_connections(void)
+{
+	int			i;
+	POOL_CONNECTION_POOL *p = pool_connection_pool;
+
+	pool_sigset_t oldmask;
+
+	POOL_SETMASK2(&BlockSig, &oldmask);
+
+	for (i = 0; i < pool_config->max_pool; i++, p++)
 	{
-		if (pool->slots[i])
-			return i;
+		if (!MAIN_CONNECTION(p))
+			continue;
+		if (!MAIN_CONNECTION(p)->sp)
+			continue;
+		if (MAIN_CONNECTION(p)->sp->user == NULL)
+			continue;
+		pool_send_frontend_exits(p);
 	}
 
-	return -1;
+	POOL_SETMASK(&oldmask);
 }
